/* ----------------------------------------------------------------------------
 * rishu_proxy.h
 *   里珠/Proxy の宣言いっぱい.
 * ----------------------------------------------------------------------------
 * Mastering programed by YAMASHINA Hio
 *
 * Copyright 2002 YMIRLINK,Inc.
 * ----------------------------------------------------------------------------
 * $Id: rishu_proxy.h,v 1.4 2002/11/09 13:38:30 hio Exp $
 * ----------------------------------------------------------------------------
 */
#ifndef RISHU_PROXY_H__
#define RISHU_PROXY_H__

/* ----------------------------------------------------------------------------
 * LPTSTR message = getLastErrorMessage();
 *   エラーメッセージの取得.
 *   取得したメッセージは LocalFree で解放してね☆ 
 */
extern LPTSTR getLastErrorMessage(void);
extern LPTSTR getErrorMessage(DWORD errcode);

/* ----------------------------------------------------------------------------
 * RishuProxy
 *   里珠/Proxy クラス.
 */
struct RishuProxy
{
  WORD   wSize;
  WORD   wVersion;
  
  /* DLL Handle */
  HANDLE hDll;
  
  /* 位置情報 */
  /* loaddir は, ANSI なら hGlobal_loaddir と同じ. */
  /*   UNICODE なら変換したもの. */
  /* dwLoaddirLen は, loaddir が決まっていたら loaddir の長さ. */
  /*   決まるまでは hGlobal_loaddir の長さ. */
  HGLOBAL hGlobal_loaddir;
  LPTSTR  loaddir;
  DWORD   dwLoaddirLen;
  
  /* 子プロセスの情報 */
  HANDLE hChlProcess;
  HANDLE hChlInput;
  HANDLE hChlOutput;
  
  /* ログ情報 */
  DWORD  dwLogBits;
  LPTSTR logfilename;
  
  /* 通信情報 */
  DWORD  msgseq;
  LPVOID readbuf;
  DWORD  dwReadBufLen;
  DWORD  dwReadBufSize;
  
  /* その他 */
  HANDLE hHeap;
  
};
typedef struct RishuProxy RishuProxy;

struct RishuProxy_Request;
typedef struct RishuProxy_Request RishuProxy_Request;

struct RishuProxy_Response;
typedef struct RishuProxy_Response RishuProxy_Response;


/* ----------------------------------------------------------------------------
 * 定数
 */
#define RishuProxy_LOG_FATAL   1
#define RishuProxy_LOG_ERROR   2
#define RishuProxy_LOG_WARN    3
#define RishuProxy_LOG_SYSTEM  4

#define RishuProxy_LOG_TRACE   5
#define RishuProxy_LOG_DEBUG   6

/* ----------------------------------------------------------------------------
 * メンバ関数 (前方宣言)
 */

/* 初期化/破棄 */
extern RishuProxy* RishuProxy_init(RishuProxy* proxy,
				   WORD wSize,
				   HANDLE hDll
				   );
extern void RishuProxy_discard(RishuProxy* proxy);

/* 排他制御 */
extern void RishuProxy_lock(RishuProxy* proxy);
extern BOOL RishuProxy_trylock(RishuProxy* proxy);
extern void RishuProxy_unlock(RishuProxy* proxy);

/* ヒープ操作 (非同期) */
extern LPVOID RishuProxy_async_myAlloc(RishuProxy* proxy,
				      SIZE_T size
				       );
extern LPVOID RishuProxy_async_myReAlloc(RishuProxy* proxy,
					LPVOID ptr,
					SIZE_T size
					 );
extern void RishuProxy_async_myFree(RishuProxy* proxy,
				    LPVOID ptr
				    );
extern SIZE_T RishuProxy_async_mySize(RishuProxy* proxy,
				      LPVOID ptr
				      );

/* ヒープ操作 (同期付き) */
extern LPVOID RishuProxy_myAlloc(RishuProxy* proxy,
				 SIZE_T size
				 );
extern LPVOID RishuProxy_myReAlloc(RishuProxy* proxy,
				   LPVOID ptr,
				   SIZE_T size
				   );
extern void RishuProxy_myFree(RishuProxy* proxy,
			      LPVOID ptr
			      );
extern SIZE_T RishuProxy_mySize(RishuProxy* proxy,
				LPVOID ptr
				);

/* loaddirからの相対ディレクトリを, loaddir を含めたものに変換. */
extern LPTSTR RishuProxy_getPath(RishuProxy* proxy,
				 LPCTSTR relativePath,
				 DWORD relativePath_len
				 );

/* 栞関数の実装 */
extern BOOL RishuProxy_on_load(RishuProxy* proxy,
			       HGLOBAL hGlobal_loaddir,
			       long loaddir_len
			       );
extern HGLOBAL RishuProxy_on_request(RishuProxy* proxy,
				     HGLOBAL hGlobal_request,
				     long* ptr_len
				     );
extern BOOL RishuProxy_on_unload(RishuProxy* proxy);

/* 子プロセスと通信 */
#define RishuProxy_getChildInput(proxy) ((proxy)->hChlInput)
extern RishuProxy_Response* RishuProxy_recvResponse(RishuProxy* proxy,
						    RishuProxy_Response* response
						    );

/* ログ機能 */
extern void RishuProxy_log(RishuProxy* proxy,
			   LPCTSTR name,
			   LPCTSTR title,
			   LPCTSTR format,
			   ...
			   );
extern void RishuProxy_vlog(RishuProxy* proxy,
			    LPCTSTR name,
			    LPCTSTR title,
			    LPCTSTR format,
			    va_list ap
			    );

/* ログが有効かの確認. */
#define RishuProxy_fatallogIsEnabled(pproxy) \
  ((pproxy)->dwLogBits & (1<<RishuProxy_LOG_FATAL))
#define RishuProxy_errorlogIsEnabled(pproxy) \
  ((pproxy)->dwLogBits & (1<<RishuProxy_LOG_ERROR))
#define RishuProxy_warnlogIsEnabled(pproxy) \
  ((pproxy)->dwLogBits & (1<<RishuProxy_LOG_WARN))
#define RishuProxy_systemlogIsEnabled(pproxy) \
  ((pproxy)->dwLogBits & (1<<RishuProxy_LOG_SYSTEM))

#define RishuProxy_tracelogIsEnabled(pproxy) \
  ((pproxy)->dwLogBits & (1<<RishuProxy_LOG_TRACE))
#define RishuProxy_debuglogIsEnabled(pproxy) \
  ((pproxy)->dwLogBits & (1<<RishuProxy_LOG_DEBUG))

/* 回答の作成 */
extern HGLOBAL RishuProxy_makeReply(HGLOBAL hGlobal,
				    long* ptr_len,
				    LPCVOID data,
				    DWORD datalen
				    );
/* 500 Internal Server Error な回答の作成. */
extern HGLOBAL RishuProxy_reply500(HGLOBAL hGlobal,
				   long* ptr_len
				   );

/* ------------------------------------------------------------------------- */
/* ----------------------------------------------------------------------------
 * 里珠/Proxy.Request
 */
struct RishuProxy_Request
{
  WORD   wSize;
  WORD   wVersion;
  /* Path */
  LPVOID path;
  DWORD  dwPathLen;
  /* 送信データ (リクエスト行は含まない) */
  LPVOID data;
  DWORD  dwDataLen;
  DWORD  dwDataSize;
};

/* ----------------------------------------------------------------------------
 * メンバ関数 (前方宣言)
 */

/* 初期化/破棄 */
extern RishuProxy_Request* RishuProxy_Request_init(RishuProxy_Request* request,
						   WORD wSize,
						   WORD version
						   );
extern void RishuProxy_Request_discard(RishuProxy_Request* request);

extern void RishuProxy_Request_setPath(RishuProxy_Request* request,
				       LPCVOID path,
				       DWORD len
				       );
extern void RishuProxy_Request_write(RishuProxy_Request* request,
				     LPCVOID data,
				     DWORD len
				     );

extern BOOL RishuProxy_Request_send(RishuProxy_Request* request,
				    RishuProxy* proxy
				    );

/* ------------------------------------------------------------------------- */
/* ----------------------------------------------------------------------------
 * 里珠/Proxy.Reponse
 */
struct RishuProxy_Response
{
  WORD   wSize;
  WORD   wVersion;
  /* 受信データ (ステータス行も含む) */
  LPVOID data;
  DWORD  dwDataLen;
  DWORD  dwDataSize;
  /* */
  LPVOID statusLine;
  DWORD  dwStatusLineLen;
  int    nStatusCode;
};

/* ----------------------------------------------------------------------------
 * メンバ関数 (前方宣言)
 */

/* 初期化/破棄 */
extern RishuProxy_Response* RishuProxy_Response_init(RishuProxy_Response* response,
						     WORD wSize
						     );
extern void RishuProxy_Response_discard(RishuProxy_Response* response);

/* 内容のクリア */
extern void RishuProxy_Response_clear(RishuProxy_Response* response);

/* ステータス取得 */
extern WORD RishuProxy_Response_getVersion(RishuProxy_Response* response);
extern int RishuProxy_Response_getStatusCode(RishuProxy_Response* response);
extern LPVOID RishuProxy_Response_getStatusLine(RishuProxy_Response* response);

/* HGLOBAL に変換 */
extern HGLOBAL RishuProxy_Response_intoGlobalMemory(RishuProxy_Response* response,
						    HGLOBAL hGlobal,
						    long* ptr_len
						    );

/* データの追加 */
extern void RishuProxy_Response_write(RishuProxy_Response* response,
				      LPCVOID data,
				      DWORD data_len
				      );

#endif
