/* ----------------------------------------------------------------------------
 * rishu_proxy_response.cpp
 *   里珠/Proxy の回答の実装.
 * ----------------------------------------------------------------------------
 * Mastering programed by YAMASHINA Hio
 *
 * Copyright 2002 YMIRLINK,Inc.
 * ----------------------------------------------------------------------------
 * $Id: rishu_proxy_response.c,v 1.1 2002/11/08 16:01:12 hio Exp $
 * ----------------------------------------------------------------------------
 */

#include <windows.h>
#include "rishu_proxy.h"
#include <stdio.h>
#include <string.h>
#include <ctype.h>

/* ----------------------------------------------------------------------------
 * 初期化.
 */
RishuProxy_Response* RishuProxy_Response_init(RishuProxy_Response* response,
					      WORD wSize
					      )
{
  if( wSize<sizeof(RishuProxy_Response) )
  {
    return NULL;
  }
  response->wSize = wSize;
  response->wVersion = 0;
  /* */
  response->data = NULL;
  response->dwDataLen = 0;
  response->dwDataSize = 0;
  /* */
  response->statusLine = NULL;
  response->dwStatusLineLen = 0;
  response->nStatusCode = 0;
  return response;
}

/* ----------------------------------------------------------------------------
 * 破棄.
 */
void RishuProxy_Response_discard(RishuProxy_Response* response)
{
  if( response->data )
  {
    HeapFree(GetProcessHeap(),0,response->data);
    response->data = NULL;
    response->dwDataLen = 0;
    response->dwDataSize = 0;
  }
  if( response->statusLine )
  {
    HeapFree(GetProcessHeap(),0,response->statusLine);
    response->statusLine = NULL;
    response->dwStatusLineLen = 0;
  }
  return;
}

/* ----------------------------------------------------------------------------
 * 内容のクリア.
 */
void RishuProxy_Response_clear(RishuProxy_Response* response)
{
  response->wVersion = 0;
  response->dwDataLen = 0;
  response->dwStatusLineLen = 0;
  response->nStatusCode = 0;  
  return;
}

/* ----------------------------------------------------------------------------
 * バージョンの取得
 */
WORD RishuProxy_Response_getVersion(RishuProxy_Response* response)
{
  if( response->wVersion )
  {
    return response->wVersion;
  }
  RishuProxy_Response_getStatusCode(response);
  return response->wVersion;
}

/* ----------------------------------------------------------------------------
 * ステータスコードの取得
 */
int RishuProxy_Response_getStatusCode(RishuProxy_Response* response)
{
  int nRes;
  int hi,low;
  int statusCode;
  
  if( response->nStatusCode )
  {
    return response->nStatusCode;
  }
  if( !response->statusLine )
  {
    RishuProxy_Response_getStatusLine(response);
    if( !response->statusLine )
    {
      return 0x0000;
    }
  }
  
  nRes = _snscanf((const char*)response->statusLine,
		  response->dwStatusLineLen,
		  "RISHU/%d.%d %d",
		  &hi, &low, &statusCode
		  );
  if( nRes!=3 )
  {
    return 0x0000;
  }
  response->wVersion = (hi<<8)|low;
  response->nStatusCode = statusCode;
  
  return statusCode;
}

/* ----------------------------------------------------------------------------
 * ステータス行の取得.
 */
LPVOID RishuProxy_Response_getStatusLine(RishuProxy_Response* response)
{
  DWORD i;
  
  if( !response->data )
  {
    return "";
  }
  if( response->statusLine )
  {
    return response->statusLine;
  }
  
  for( i=0; i<response->dwDataLen; ++i )
  {
    if( ((LPCBYTE)response->data)[i]=='\n' )
    {
      LPBYTE statusLine;
      if( i && ((LPCBYTE)response->data)[i-1]=='\r' )
      {
	--i;
      }
      statusLine = HeapAlloc(GetProcessHeap(),0,i+1);
      if( !statusLine )
      {
	return "";
      }
      CopyMemory(statusLine,response->data,i);
      statusLine[i] = '\0';
      response->statusLine = statusLine;
      response->dwStatusLineLen = i;
      return response->statusLine;
    }
  }
  return "";
}

/* ----------------------------------------------------------------------------
 * HGLOBAL に変換.
 */
HGLOBAL RishuProxy_Response_intoGlobalMemory(RishuProxy_Response* response,
					     HGLOBAL hGlobal,
					     long* ptr_len)
{
  LPCBYTE ptr;
  const LPCBYTE tail = ((LPCBYTE)response->data)+response->dwDataLen;
  
  if( !response->data )
  {
    return RishuProxy_reply500(hGlobal,ptr_len);
  }
  
  ptr = response->data;
  /* ステータス行を解析. */
  RishuProxy_Response_getStatusLine(response);
  ptr += response->dwStatusLineLen;
  if( ptr >= tail ) return RishuProxy_reply500(hGlobal,ptr_len);
  if( *ptr=='\r' )
  {
    ++ptr;
    if( ptr >= tail ) return RishuProxy_reply500(hGlobal,ptr_len);
  }
  if( *ptr=='\n' )
  {
    ++ptr;
    if( ptr >= tail ) return RishuProxy_reply500(hGlobal,ptr_len);
  }
  
  /* Original-Response: の除去. */
  {
    BYTE cmp[] = "Original-Response:";
    DWORD cmplen = sizeof(cmp)-1;
    if( ptr+cmplen<tail )
    {
      if( strncmp(ptr,cmp,cmplen)==0 )
      {
        ptr += cmplen;
      }else
      { /* strncasecmp の代用. */
	int i;
	for( i=0; ; ++i )
	{
	  if( i==cmplen )
	  {
	    ptr += i;
	    break;
	  }
	  if( cmp[i]!=ptr[i] && tolower(cmp[i])!=tolower(ptr[i]) )
	  {
	    break;
	  }
	}
      }
    }
    while( isspace(*ptr) && ++ptr<tail );
  }
  
  return hGlobal;
}

/* ----------------------------------------------------------------------------
 * データの追加
 */
void RishuProxy_Response_write(RishuProxy_Response* response,
			       LPCVOID data,
			       DWORD data_len
			       )
{
  if( response->dwDataSize-response->dwDataLen<data_len )
  {
    if( !response->data )
    {
      response->data = HeapAlloc(GetProcessHeap(),0,data_len);
      if( !response->data )
      {
	return;
      }
    }else
    {
      register const HGLOBAL olddata = response->data;
      response->data = HeapReAlloc(GetProcessHeap(),0,olddata,response->dwDataLen+data_len);
      if( !response->data )
      {
	response->data = olddata;
	return;
      }
    }
    response->dwDataSize = HeapSize(GetProcessHeap(),0,response->data);
  }
  CopyMemory((LPBYTE)response->data+response->dwDataLen,data,data_len);
  response->dwDataLen   += data_len;
  return;
}

